@@ -1,3 +1,4 @@
+
 # fmt: off
 from __future__ import annotations
 
@@ -9,89 +10,13 @@ import sys
 import typing
 from collections.abc import Generator
 from contextlib import contextmanager
-from functools import wraps
+from functools import cache, wraps
 
 from voluptuous import error as er
 from voluptuous.error import Error
 
 # fmt: on
 
-"""Schema validation for Python data structures.
-
-Given eg. a nested data structure like this:
-
-    {
-        'exclude': ['Users', 'Uptime'],
-        'include': [],
-        'set': {
-            'snmp_community': 'public',
-            'snmp_timeout': 15,
-            'snmp_version': '2c',
-        },
-        'targets': {
-            'localhost': {
-                'exclude': ['Uptime'],
-                'features': {
-                    'Uptime': {
-                        'retries': 3,
-                    },
-                    'Users': {
-                        'snmp_community': 'monkey',
-                        'snmp_port': 15,
-                    },
-                },
-                'include': ['Users'],
-                'set': {
-                    'snmp_community': 'monkeys',
-                },
-            },
-        },
-    }
-
-A schema like this:
-
-    >>> settings = {
-    ...   'snmp_community': str,
-    ...   'retries': int,
-    ...   'snmp_version': All(Coerce(str), Any('3', '2c', '1')),
-    ... }
-    >>> features = ['Ping', 'Uptime', 'Http']
-    >>> schema = Schema({
-    ...    'exclude': features,
-    ...    'include': features,
-    ...    'set': settings,
-    ...    'targets': {
-    ...      'exclude': features,
-    ...      'include': features,
-    ...      'features': {
-    ...        str: settings,
-    ...      },
-    ...    },
-    ... })
-
-Validate like so:
-
-    >>> schema({
-    ...   'set': {
-    ...     'snmp_community': 'public',
-    ...     'snmp_version': '2c',
-    ...   },
-    ...   'targets': {
-    ...     'exclude': ['Ping'],
-    ...     'features': {
-    ...       'Uptime': {'retries': 3},
-    ...       'Users': {'snmp_community': 'monkey'},
-    ...     },
-    ...   },
-    ... }) == {
-    ...   'set': {'snmp_version': '2c', 'snmp_community': 'public'},
-    ...   'targets': {
-    ...     'exclude': ['Ping'],
-    ...     'features': {'Uptime': {'retries': 3},
-    ...                  'Users': {'snmp_community': 'monkey'}}}}
-    True
-"""
-
 # options for extra keys
 PREVENT_EXTRA = 0  # any extra key not in schema will raise an error
 ALLOW_EXTRA = 1  # extra keys not in schema will be included in output
@@ -208,7 +133,7 @@ class Schema(object):
             - Any value other than the above defaults to
               :const:`~voluptuous.PREVENT_EXTRA`
         """
-        self.schema = schema
+        self.schema: typing.Any = schema
         self.required = required
         self.extra = int(extra)  # ensure the value is an integer
         self._compiled = self._compile(schema)
@@ -435,11 +360,11 @@ class Schema(object):
 
                     break
                 else:
-                    if error:
-                        errors.append(error)
-                    elif remove_key:
+                    if remove_key:
                         # remove key
                         continue
+                    elif error:
+                        errors.append(error)
                     elif self.extra == ALLOW_EXTRA:
                         out[key] = value
                     elif self.extra != REMOVE_EXTRA:
@@ -1026,16 +951,19 @@ class Marker(object):
     introspected by any external tool, for example to generate schema documentation.
     """
 
+    __slots__ = ('schema', '_schema', 'msg', 'description', '__hash__')
+
     def __init__(
         self,
         schema_: Schemable,
         msg: typing.Optional[str] = None,
-        description: typing.Optional[str] = None,
+        description: typing.Any | None = None,
     ) -> None:
-        self.schema = schema_
+        self.schema: typing.Any = schema_
         self._schema = Schema(schema_)
         self.msg = msg
         self.description = description
+        self.__hash__ = cache(lambda: hash(schema_))  # type: ignore[method-assign]
 
     def __call__(self, v):
         try:
@@ -1056,9 +984,6 @@ class Marker(object):
             return self.schema < other.schema
         return self.schema < other
 
-    def __hash__(self):
-        return hash(self.schema)
-
     def __eq__(self, other):
         return self.schema == other
 
@@ -1093,8 +1018,8 @@ class Optional(Marker):
         self,
         schema: Schemable,
         msg: typing.Optional[str] = None,
-        default=UNDEFINED,
-        description: typing.Optional[str] = None,
+        default: typing.Any = UNDEFINED,
+        description: typing.Any | None = None,
     ) -> None:
         super(Optional, self).__init__(schema, msg=msg, description=description)
         self.default = default_factory(default)
@@ -1141,7 +1066,7 @@ class Exclusive(Optional):
         schema: Schemable,
         group_of_exclusion: str,
         msg: typing.Optional[str] = None,
-        description: typing.Optional[str] = None,
+        description: typing.Any | None = None,
     ) -> None:
         super(Exclusive, self).__init__(schema, msg=msg, description=description)
         self.group_of_exclusion = group_of_exclusion
@@ -1194,8 +1119,8 @@ class Inclusive(Optional):
         schema: Schemable,
         group_of_inclusion: str,
         msg: typing.Optional[str] = None,
-        description: typing.Optional[str] = None,
-        default=UNDEFINED,
+        description: typing.Any | None = None,
+        default: typing.Any = UNDEFINED,
     ) -> None:
         super(Inclusive, self).__init__(
             schema, msg=msg, default=default, description=description
@@ -1222,8 +1147,8 @@ class Required(Marker):
         self,
         schema: Schemable,
         msg: typing.Optional[str] = None,
-        default=UNDEFINED,
-        description: typing.Optional[str] = None,
+        default: typing.Any = UNDEFINED,
+        description: typing.Any | None = None,
     ) -> None:
         super(Required, self).__init__(schema, msg=msg, description=description)
         self.default = default_factory(default)
@@ -1244,6 +1169,15 @@ class Remove(Marker):
     [1, 2, 3, 5, '7']
     """
 
+    def __init__(
+        self,
+        schema_: Schemable,
+        msg: typing.Optional[str] = None,
+        description: typing.Any | None = None,
+    ) -> None:
+        super().__init__(schema_, msg, description)
+        self.__hash__ = cache(lambda: object.__hash__(self))  # type: ignore[method-assign]
+
     def __call__(self, schema: Schemable):
         super(Remove, self).__call__(schema)
         return self.__class__
@@ -1251,9 +1185,6 @@ class Remove(Marker):
     def __repr__(self):
         return "Remove(%r)" % (self.schema,)
 
-    def __hash__(self):
-        return object.__hash__(self)
-
 
 def message(
     default: typing.Optional[str] = None,
